import unittest

import pyisg

TEMPLATE = {
    "comment": "",
    "header": {
        "model_name": "EXAMPLE",
        "model_year": "2020",
        "model_type": "gravimetric",
        "data_type": "geoid",
        "data_units": "meters",
        "data_format": "grid",
        "data_ordering": "N-to-S, W-to-E",
        "ref_ellipsoid": "GRS80",
        "ref_frame": "ITRF2014",
        "height_datum": None,
        "tide_system": "mean-tide",
        "coord_type": "geodetic",
        "coord_units": "dms",
        "map_projection": None,
        "EPSG_code": "7912",
        "lat_min": {"degree": 39, "minutes": 50, "second": 0},
        "lat_max": {"degree": 41, "minutes": 10, "second": 0},
        "lon_min": {"degree": 119, "minutes": 50, "second": 0},
        "lon_max": {"degree": 121, "minutes": 50, "second": 0},
        "delta_lat": {"degree": 0, "minutes": 20, "second": 0},
        "delta_lon": {"degree": 0, "minutes": 20, "second": 0},
        "nrows": 4,
        "ncols": 6,
        "nodata": -9999.0,
        "creation_date": {"year": 2020, "month": 5, "day": 31},
        "ISG_format": "2.0",
    },
    "data": [
        [30.1234, 31.2222, 32.3456, 33.4444, 34.5678, 36.6666],
        [41.1111, 42.2345, 43.3333, 44.4567, 45.5555, 46.6789],
        [51.4321, 52.9753, 53.6543, 54.8642, None, None],
        [61.9999, 62.8888, 63.7777, 64.6666, None, None],
    ],
}


class TestTypeError(unittest.TestCase):
    def test_comment(self):
        obj = {
            "comment": 1,
            "header": {
                "model_name": "EXAMPLE",
                "model_year": "2020",
                "model_type": "gravimetric",
                "data_type": "geoid",
                "data_units": "meters",
                "data_format": "grid",
                "data_ordering": "N-to-S, W-to-E",
                "ref_ellipsoid": "GRS80",
                "ref_frame": "ITRF2014",
                "height_datum": None,
                "tide_system": "mean-tide",
                "coord_type": "geodetic",
                "coord_units": "dms",
                "map_projection": None,
                "EPSG_code": "7912",
                "lat_min": {"degree": 39, "minutes": 50, "second": 0},
                "lat_max": {"degree": 41, "minutes": 10, "second": 0},
                "lon_min": {"degree": 119, "minutes": 50, "second": 0},
                "lon_max": {"degree": 121, "minutes": 50, "second": 0},
                "delta_lat": {"degree": 0, "minutes": 20, "second": 0},
                "delta_lon": {"degree": 0, "minutes": 20, "second": 0},
                "nrows": 4,
                "ncols": 6,
                "nodata": -9999.0,
                "creation_date": {"year": 2020, "month": 5, "day": 31},
                "ISG_format": "2.0",
            },
            "data": [
                [30.1234, 31.2222, 32.3456, 33.4444, 34.5678, 36.6666],
                [41.1111, 42.2345, 43.3333, 44.4567, 45.5555, 46.6789],
                [51.4321, 52.9753, 53.6543, 54.8642, None, None],
                [61.9999, 62.8888, 63.7777, 64.6666, None, None],
            ],
        }

        with self.assertRaises(pyisg.SerializeError) as cm:
            pyisg.dumps(obj)
        self.assertEqual(cm.exception.args, ("unexpected type on `comment`, expected str | None",))

    def test_creation_date(self):
        obj = {
            "header": {
                "model_name": "EXAMPLE",
                "model_year": "2020",
                "model_type": "gravimetric",
                "data_type": "geoid",
                "data_units": "meters",
                "data_format": "grid",
                "data_ordering": "N-to-S, W-to-E",
                "ref_ellipsoid": "GRS80",
                "ref_frame": "ITRF2014",
                "height_datum": None,
                "tide_system": "mean-tide",
                "coord_type": "geodetic",
                "coord_units": "dms",
                "map_projection": None,
                "EPSG_code": "7912",
                "lat_min": {"degree": 39, "minutes": 50, "second": 0},
                "lat_max": {"degree": 41, "minutes": 10, "second": 0},
                "lon_min": {"degree": 119, "minutes": 50, "second": 0},
                "lon_max": {"degree": 121, "minutes": 50, "second": 0},
                "delta_lat": {"degree": 0, "minutes": 20, "second": 0},
                "delta_lon": {"degree": 0, "minutes": 20, "second": 0},
                "nrows": 4,
                "ncols": 6,
                "nodata": -9999.0,
                "creation_date": {"year": 100000, "month": 5, "day": 31},
                "ISG_format": "2.0",
            },
            "data": [
                [30.1234, 31.2222, 32.3456, 33.4444, 34.5678, 36.6666],
                [41.1111, 42.2345, 43.3333, 44.4567, 45.5555, 46.6789],
                [51.4321, 52.9753, 53.6543, 54.8642, None, None],
                [61.9999, 62.8888, 63.7777, 64.6666, None, None],
            ],
        }

        with self.assertRaises(pyisg.SerializeError) as cm:
            pyisg.dumps(obj)
        self.assertEqual(
            cm.exception.args,
            (
                "unexpected type on `creation_date`, expected { year: int (u16), month: int (u8), day: int (u8) } | None",
            ),
        )


class TestDeError(unittest.TestCase):
    def test_header(self):
        obj = {
            "comment": "",
            "data": [
                [30.1234, 31.2222, 32.3456, 33.4444, 34.5678, 36.6666],
                [41.1111, 42.2345, 43.3333, 44.4567, 45.5555, 46.6789],
                [51.4321, 52.9753, 53.6543, 54.8642, None, None],
                [61.9999, 62.8888, 63.7777, 64.6666, None, None],
            ],
        }

        with self.assertRaises(pyisg.SerializeError) as cm:
            pyisg.dumps(obj)
        self.assertEqual(cm.exception.args, ("missing `header`",))


if __name__ == "__main__":
    unittest.main()



"""Provides definition of types."""

from __future__ import annotations

from typing import Literal, TypeAlias, TypedDict

__all__ = [
    "ISGFormatType",
    #
    "HeaderType",
    #
    "ModelTypeType",
    "DataTypeType",
    "DataUnitsType",
    "DataFormatType",
    "DataOrderingType",
    "TideSystemType",
    "CoordTypeType",
    "CoordUnitsType",
    "CreationDateType",
    "DmsCoordType",
    #
    "SparseData",
    "GridData",
]

ModelTypeType: TypeAlias = Literal["gravimetric", "geometric", "hybrid"]
DataTypeType: TypeAlias = Literal["geoid", "quasi-geoid"]
DataUnitsType: TypeAlias = Literal["meters", "feet"]
DataFormatType: TypeAlias = Literal["grid", "sparse"]
DataOrderingType: TypeAlias = Literal["N-to-S, W-to-E", "lat, lon, N", "east, north, N", "N", "zeta"]
TideSystemType: TypeAlias = Literal["tide-free", "mean-tide", "zero-tide"]
CoordTypeType: TypeAlias = Literal["geodetic", "projected"]
CoordUnitsType: TypeAlias = Literal["dms", "deg", "meters", "feet"]


class CreationDateType(TypedDict):
    """Type of creation date."""

    year: int
    month: int
    day: int


class DmsCoordType(TypedDict):
    """Type of DMS coordinate."""

    degree: int
    minutes: int
    second: int


class HeaderType(TypedDict):
    """Type of Header dict."""

    model_name: str | None
    model_year: str | None
    model_type: ModelTypeType | None
    data_type: DataTypeType | None
    data_units: DataUnitsType | None
    data_format: DataFormatType | None
    data_ordering: DataOrderingType | None
    ref_ellipsoid: str | None
    ref_frame: str | None
    height_datum: str | None
    tide_system: TideSystemType | None
    coord_type: CoordTypeType
    coord_units: CoordUnitsType
    map_projection: str | None
    EPSG_code: str | None
    lat_min: float | DmsCoordType | None
    lat_max: float | DmsCoordType | None
    north_min: float | DmsCoordType | None
    north_max: float | DmsCoordType | None
    lon_min: float | DmsCoordType | None
    lon_max: float | DmsCoordType | None
    east_min: float | DmsCoordType | None
    east_max: float | DmsCoordType | None
    delta_lat: float | DmsCoordType | None
    delta_lon: float | DmsCoordType | None
    delta_north: float | DmsCoordType | None
    delta_east: float | DmsCoordType | None
    nrows: int
    ncols: int
    nodata: float | None
    creation_date: CreationDateType | None
    ISG_format: str


SparseData: TypeAlias = list[tuple[float | DmsCoordType, float | DmsCoordType, float]]
GridData: TypeAlias = list[list[float | None]]


class ISGFormatType(TypedDict):
    """Type of ISG data dict."""

    comment: str
    header: HeaderType
    data: GridData | SparseData



import unittest

from pathlib import Path
from pprint import pprint

import pyisg


class Identity(unittest.TestCase):
    maxDiff = None

    def test_example_1(self):
        p = Path("tests/rsc/example.1.isg")
        expected = p.read_text()

        obj = pyisg.loads(expected)
        actual = pyisg.dumps(obj)

        self.assertEqual(expected, actual)

    def test_example_2(self):
        p = Path("tests/rsc/example.2.isg")
        expected = p.read_text()

        obj = pyisg.loads(expected)
        actual = pyisg.dumps(obj)

        self.assertEqual(expected, actual)

    @unittest.skip("pyo3 bug?")
    def test_example_3(self):
        p = Path("tests/rsc/example.3.isg")
        expected = p.read_text()

        obj = pyisg.loads(expected)
        actual = pyisg.dumps(obj)

        self.assertEqual(expected, actual)


if __name__ == "__main__":
    unittest.main()



"""Provides serialize/deserialize API of ISG 2.0 format."""

from __future__ import annotations

from typing import Final, TextIO

from . import types
from .types import ISGFormatType

try:
    from . import pyisg as rsimpl
except ImportError:
    import warnings

    warnings.warn("pyisg: not supported platform", stacklevel=1)
    raise

__version__: Final = "0.1.0"

__all__ = [
    "types",
    #
    "ISGFormatType",
    #
    "loads",
    "load",
    "dumps",
    "dump",
    #
    "SerializeError",
    "DeserializeError",
]


def loads(s: str) -> ISGFormatType:
    """Deserialize ISG 2.0 format :obj:`str` to :obj:`dict`.

    Args:
        s: ISG 2.0 format :obj:`str`

    Returns:
        dict of ISG data

    Raises:
        DeserializeError: deserialization failed
    """
    try:
        return rsimpl.loads(s)
    except Exception as e:
        raise DeserializeError(*e.args) from None


def load(fp: TextIO) -> ISGFormatType:
    """Deserialize ISG 2.0 file-like obj to :obj:`dict`.

    Args:
        fp: file-like obj of ISG 2.0 format data

    Returns:
        dict of ISG data

    Raises:
        DeserializeError: deserialization failed
    """
    return loads(fp.read())


def dumps(obj: ISGFormatType) -> str:
    """Serialize ISG 2.0 formatted str into :obj:`str`.

    Args:
        obj: dict of ISG data

    Raises:
        SerializeError: serialization failed
    """
    try:
        return rsimpl.dumps(obj)
    except Exception as e:
        raise SerializeError(*e.args) from None


def dump(obj: ISGFormatType, fp: TextIO) -> int:
    """Serialize ISG 2.0 formatted str into file-like obj.

    Args:
        obj: dict of ISG data
        fp: output file-like obj

    Returns:
        return value of fp.write

    Raises:
        SerializeError: serialization failed
    """
    return fp.write(dumps(obj))


class SerializeError(ValueError):
    """Error of :func:`dump` and :func:`dumps`."""

    pass


class DeserializeError(ValueError):
    """Error of :func:`load` and :func:`loads`."""

    pass
